<?php
// $Id: libcdn2.inc,v 1.3.2.2 2008/12/20 17:58:34 akalsey Exp $

/* TODO db_next_id() is gone, and replaced as db_last_insert_id()
   Since db_next_id() introduce some problems, and the use of this function
   can be replaced by database level auto increment handling, db_next_id()
   is now gone and replaced as db_last_insert_id() with help of serial type
   under Schema API (check out http://drupal.org/node/149176 for more details).
   Please refer to drupal_write_record() as demonstration. */

/* TODO FormAPI image buttons are now supported.
   FormAPI now offers the 'image_button' element type, allowing developers to
   use icons or other custom images in place of traditional HTML submit buttons.

$form['my_image_button'] = array(
  '#type'         => 'image_button',
  '#title'        => t('My button'),
  '#return_value' => 'my_data',
  '#src'          => 'my/image/path.jpg',
); */

/* TODO Node previews and adding form fields to the node form.
   There is a subtle but important difference in the way node previews (and other
   such operations) are carried out when adding or editing a node. With the new
   Forms API, the node form is handled as a multi-step form. When the node form
   is previewed, all the form values are submitted, and the form is rebuilt with
   those form values put into $form['#node']. Thus, form elements that are added
   to the node form will lose any user input unless they set their '#default_value'
   elements using this embedded node object. */

/* TODO New user_mail_tokens() method may be useful.
   user.module now provides a user_mail_tokens() function to return an array
   of the tokens available for the email notification messages it sends when
   accounts are created, activated, blocked, etc. Contributed modules that
   wish to make use of the same tokens for their own needs are encouraged
   to use this function. */

/* TODO
   There is a new hook_watchdog in core. This means that contributed modules
   can implement hook_watchdog to log Drupal events to custom destinations.
   Two core modules are included, dblog.module (formerly known as watchdog.module),
   and syslog.module. Other modules in contrib include an emaillog.module,
   included in the logging_alerts module. See syslog or emaillog for an
   example on how to implement hook_watchdog.
function example_watchdog($log = array()) {
  if ($log['severity'] == WATCHDOG_ALERT) {
    mysms_send($log['user']->uid,
      $log['type'],
      $log['message'],
      $log['variables'],
      $log['severity'],
      $log['referer'],
      $log['ip'],
      format_date($log['timestamp']));
  }
} */

/* TODO Implement the hook_theme registry. Combine all theme registry entries
   into one hook_theme function in each corresponding module file.
function libcdn2_theme() {
  return array(
  );
}; */


/* TODO
   An argument for replacements has been added to format_plural(),
   escaping and/or theming the values just as done with t().*/

/*
 * @file libcdn2.inc
 */
module_load_include('hp"', 'libcdn2', '');

class ErrorHandlingSoapClient {
  private $soapImpl;

  public function __construct($soapImpl) {
    $this->soapImpl = $soapImpl;
  }

  public function __call($function, $args) {
    try {
      return call_user_func_array(array($this->soapImpl, $function), $args);
    } catch (Exception $e) {
      return array('error' => $e->getMessage());
    } 
  }
}

interface CDN2SoapClient {
  public function getVideoToken($formats);
  public function getFormatPresets();
  public function getAssetsForVideoToken($videoToken);
  public function getVideosForSite();
  public function getVideoUploadStatus($videoToken);
  public function reencodeVideo($videoToken, $formats = array());
  public function deleteVideo($videoToken);
}

class CDN2SoapClientImpl implements CDN2SoapClient {
  static $instance;
  private $_siteToken;
  private $_secretKey;
  private $_hmac;
  private $_endpointUrl;

  public static function getInstance($endPointUrl, $siteToken, $secretKey) {
    if (self::$instance == NULL) {
            self::$instance = new CDN2SoapClientImpl($endPointUrl, $siteToken, $secretKey);
    }
    return self::$instance; 
  }
  
  private function CDN2SoapClientImpl($endPointUrl, $siteToken, $secretKey) {
    $this->_endPointUrl = $endPointUrl;
    $this->_siteToken = $siteToken;
    $this->_secretKey = $secretKey;
    $this->_hmac = new CDN2_Crypt_HMAC($this->_secretKey);
    $this->_soapClient = new SoapClient($this->_endPointUrl, array('classmap' => array('authenticationToken' => 'CDN2AuthenticationToken'), 'trace' => 1));
    
  }

  public function getVideoToken($formats) {
    return $this->_soapClient->getVideoToken(array('authenticationToken' => $this->createDigestAuthenticationToken(), 'siteToken' => $this->_siteToken, 'formats' => $formats))->return;
  }

  public function getFormatPresets() {
    $result = $this->_soapClient->getFormatPresets(array('authenticationToken' => $this->createDigestAuthenticationToken()))->return;
    return $result;
  }

  public function getAssetsForVideoToken($videoToken) {
    return $this->_soapClient->getAssetsForVideoToken(array('authenticationToken' => $this->createDigestAuthenticationToken(), 'videoToken' => $videoToken))->return;
  }

  public function getVideosForSite() {
    return $this->_soapClient->getVideosForSite(array('authenticationToken' => $this->createDigestAutheticationToken()))->return;
  }
  public function getVideoUploadStatus($videoToken) {
    return $this->_soapClient->getVideoUploadStatus(array('authenticationToken' => $this->createDigestAuthenticationToken(), 'videoToken'=> $videoToken))->return;
  }
  public function reencodeVideo($videoToken, $formats = array()) {
    return $this->_soapClient->reencodeVideo(array('authenticationToken' => $this->createDigestAuthenticationToken(), 'videoToken' => $videoToken, 'formats' => $formats))->return;
  }
  public function deleteVideo($videoToken) {
    return $this->_soapClient->reencodeVideo(array('authenticationToken' => $this->createDigestAuthenticationToken(), 'videoToken' => $videoToken))->return;
  }
  private function createDigestAuthenticationToken() {
    $timeStamp = date(DATE_RFC3339);
    $this->_hmac->setKey($this->_secretKey);
    $this->_hmac->setFunction("sha1");
    
    $calculatedHash = $this->_hmac->hash($this->_siteToken . $timeStamp);
    $authToken = new CDN2AuthenticationToken();
    $authToken->siteToken = $this->_siteToken;
    $authToken->timeStamp = $timeStamp;
    $authToken->calculatedHash = $calculatedHash;

    return new SoapVar($authToken, SOAP_ENC_OBJECT, 'cdn2AuthenticationToken', 'http://xml.radiantcdn.com/transcode');
  }
}

class CDN2AuthenticationToken {
  public $siteToken;
  public $timeStamp;
  public $calculatedHash;
  
}

class CDN2_Crypt_HMAC extends Crypt_HMAC {
    /** override default HMAC support since it uses a hex string and we want base64 */
    function hash($data) {
        $func = $this->_func;
        $rawHash = $func($this->_opad . pack($this->_pack, $func($this->_ipad . $data)), true);
        return base64_encode($rawHash);
    }

}
